@@ -9,7 +9,7 @@
 }
 
 .abbrevOrganismName <- function(organism){
-  spc <- unlist(strsplit(organism, "_"))
+  spc <- unlist(strsplit(organism, "_|[[:space:]]"))
   ## this assumes a binomial nomenclature has been maintained.
   return(paste0(substr(spc[[1]], 1, 1), spc[[2]]))
 }
@@ -25,10 +25,9 @@
 
 .makeObjectName <- function(pkgName){
   strs <- unlist(strsplit(pkgName, "\\."))
-  paste(c(strs[2:length(strs)],strs[1]), collapse="_")
+  paste(c(strs[2:length(strs)], strs[1]), collapse="_")
 }
 
-
 ## retrieve Ensembl data
 ## save all files to local folder.
 ## returns the path where files have been saved to.
@@ -64,7 +63,6 @@ fetchTablesFromEnsembl <- function(version, ensemblapi, user="anonymous",
         stop("Something went wrong! I'm missing some of the txt files the perl script should have generated.")
 }
 
-
 ####
 ##
 ## create a SQLite database containing the information defined in the txt files.
@@ -91,8 +89,7 @@ makeEnsemblSQLiteFromTables <- function(path=".", dbname){
     ##substring(species, 1, 1) <- toupper(substring(species, 1, 1))
     if(missing(dbname)){
         dbname <- paste0(
-            "EnsDb.",substring(species, 1, 1),
-            unlist(strsplit(species, split="_"))[ 2 ], ".v",
+            "EnsDb.", .abbrevOrganismName(species), ".v",
             info[ info$name=="ensembl_version", "value" ], ".sqlite")
     }
     con <- dbConnect(dbDriver("SQLite"), dbname=dbname)
@@ -253,7 +250,6 @@ makeEnsemblSQLiteFromTables <- function(path=".", dbname){
     return(TRUE)
 }
 
-
 ####
 ## the function that creates the annotation package.
 ## ensdb should be a connection to an SQLite database, or a character string...
@@ -311,7 +307,6 @@ makeEnsembldbPackage <- function(ensdb,
     file.copy(ensdbfile, to=db_path)
 }
 
-
 ####
 ## function to create a EnsDb object (or rather the SQLite database) from
 ## a Ensembl GTF file.
@@ -524,8 +519,6 @@ ensDbFromAH <- function(ah, outfile, path, organism, genomeVersion, version) {
     return(res)
 }
 
-
-
 ####============================================================
 ##
 ##  ensDbFromGff
@@ -680,8 +673,6 @@ ensDbFromGff <- function(gff, outfile, path, organism, genomeVersion,
     return(dbname)
 }
 
-
-
 #### build a EnsDb SQLite database from the GRanges.
 ## we can however not get all of the information from the GRanges (yet), for example,
 ## the seqinfo might not be available in all GRanges objects. Also, there is no way
@@ -985,7 +976,6 @@ ensDbFromGRanges <- function(x, outfile, path, organism, genomeVersion,
     return(dbname)
 }
 
-
 ## helper function that checks that the gene, transcript and exon data in the
 ## EnsDb database is correct (i.e. transcript within gene coordinates, exons within
 ## transcript coordinates, cds within transcript)
@@ -1045,7 +1035,6 @@ checkValidEnsDb <- function(x){
     return(TRUE)
 }
 
-
 ############################################################
 ##' Fetch chromosome sequence lengths from Ensembl.
 ##' @param organism The organism. Has to be in the form "Homo sapiens"
@@ -1072,7 +1061,6 @@ tryGetSeqinfoFromEnsembl <- function(organism, ensemblVersion, seqnames,
     return(tmp)
 }
 
-
 buildMetadata <- function(organism="", ensemblVersion="", genomeVersion="",
                           host="", sourceFile=""){
     MetaData <- data.frame(matrix(ncol=2, nrow=11))
@@ -1151,7 +1139,6 @@ compareEnsDbs <- function(x, y){
     return(Messages)
 }
 
-
 compareChromosomes <- function(x, y){
     Ret <- "OK"
     cat("\nComparing chromosome data:\n")
@@ -1244,9 +1231,13 @@ compareGenes <- function(x, y){
     cat(paste0( " Gene names: (",same,
                ") identical, (", different, ") different.\n" ))
     ## entrezid
-    same <- length(
-        which(genesX[inboth]$entrezid == genesY[inboth]$entrezid)
-        )
+    funny <- function(z) {
+        if (all(is.na(z))) ""
+        else paste(sort(z))
+    }
+    eidsX <- unlist(lapply(genesX[inboth]$entrezid, funny))
+    eidsY <- unlist(lapply(genesY[inboth]$entrezid, funny))
+    same <- length(which(eidsX == eidsY))
     different <- length(inboth) - same
     if(different > 0 & Ret!="ERROR")
         Ret <- "WARN"
@@ -1473,9 +1464,11 @@ isEnsemblFileName <- function(x){
     ## Well, can not think of any other torture... let's assume it's OK.
     return(TRUE)
 }
+
 organismFromGtfFileName <- function(x){
     return(elementFromEnsemblFilename(x, 1))
 }
+
 ####============================================================
 ##  ensemblVersionFromGtfFileName
 ##
@@ -1495,6 +1488,7 @@ ensemblVersionFromGtfFileName <- function(x){
     }
     return(NA)
 }
+
 ####============================================================
 ##  genomeVersionFromGtfFileName
 ##
