@@ -289,10 +289,9 @@ setMethod("mapIds", "EnsDb", function(x, keys, column, keytype, ..., multiVals)
         keytype <- NULL
     if(missing(multiVals))
         multiVals <- NULL
-    return(.mapIds(x = x, keys = keys, column = column, keytype = keytype,
-                   multiVals = multiVals, ...))
+    .mapIds(x = x, keys = keys, column = column, keytype = keytype,
+            multiVals = multiVals, ...)
 })
-## Other methods: saveDb, species, dbfile, dbconn, taxonomyId
 .mapIds <- function(x, keys = NULL, column = NULL, keytype = NULL, ...,
                     multiVals = NULL) {
     if (is.null(keys))
@@ -318,42 +317,48 @@ setMethod("mapIds", "EnsDb", function(x, keys, column, keytype, ..., multiVals)
         keytype <- NULL
     }
     res <- select(x, keys = keys, columns = columns, keytype = keytype)
-    if(nrow(res) == 0)
-        return(character())
+    ## if(nrow(res) == 0)
+    ##     return(character())
     ## Handling multiVals.
-    if(is.null(multiVals))
+    if (is.null(multiVals))
         multiVals <- "first"
-    if(is(multiVals, "function"))
+    if (is(multiVals, "function"))
         stop("Not yet implemented!")
-    if(is.character(keys)){
+    if (is.character(keys)) {
         theNames <- keys
-    }else{
+    } else {
         theNames <- unique(res[, 1])
     }
     switch(multiVals,
-           first={
+           first = {
                vals <- res[match(theNames, res[, 1]), 2]
                names(vals) <- theNames
-               return(vals)
+               not_mapped <- sum(is.na(vals))
            },
-           list={
+           list = {
                vals <- split(res[, 2], f=factor(res[, 1], levels=unique(theNames)))
-               return(vals)
+               not_mapped <- sum(lengths(vals) == 0)
            },
-           filter={
-               vals <- split(res[, 2], f=factor(res[, 1], levels=unique(theNames)))
-               vals <- vals[unlist(lapply(vals, length)) == 1]
-               return(unlist(vals))
+           filter = {
+               vals <- split(res[, 2], f = factor(res[, 1],
+                                                  levels = unique(theNames)))
+               vals <- unlist(vals[lengths(vals) == 1])
+               if (length(vals)) return(vals) else return(setNames(nm = character()))
            },
-           asNA={
+           asNA = {
                ## Split the vector, set all those with multi mappings NA.
                vals <- split(res[, 2], f=factor(res[, 1], levels=unique(theNames)))
-               vals[unlist(lapply(vals, length)) > 1] <- NA
-               return(unlist(vals))
+               not_mapped <- sum(lengths(vals) == 0)
+               vals[unlist(lapply(vals, length)) > 1] <- NA_character_
+               vals <- unlist(vals)
            },
-           CharacterList={
-               f <- factor(res[, 1], levels=unique(theNames))
+           CharacterList = {
+               f <- factor(res[, 1], levels = unique(theNames))
                vals <- splitAsList(res[, 2], f=f)
-               return(vals)
+               not_mapped <- sum(lengths(vals) == 0)
            })
+    if (not_mapped)
+        warning("Unable to map ", not_mapped, " of ", length(theNames),
+                " requested IDs.", call. = FALSE)
+    vals
 }
