@@ -1,5 +1,7 @@
 #!/usr/bin/perl
 #####################################
+## version 0.3.4: * Add columns gene_id_version and tx_id_version to the gene
+##                  and transcript tables.
 ## version 0.3.3: * Write the species' scientific name to the Organism metadata
 ##                  field.
 ## version 0.3.1: * Add ens_counts.txt with the total counts of genes, tx, exons
@@ -30,7 +32,7 @@ use Bio::EnsEMBL::ApiVersion;
 use Bio::EnsEMBL::Registry;
 ## unification function for arrays
 use List::MoreUtils qw/ uniq /;
-my $script_version = "0.3.3";
+my $script_version = "0.3.4";
 my $min_tsl_version = 87;   ## The minimal required Ensembl version providing support for the tsl method.
 
 ## connecting to the ENSEMBL data base
@@ -130,10 +132,10 @@ print $infostring;
 
 ## preparing output files:
 open(GENE , ">ens_gene.txt");
-print GENE "gene_id\tgene_name\tgene_biotype\tgene_seq_start\tgene_seq_end\tseq_name\tseq_strand\tseq_coord_system\tdescription\n";
+print GENE "gene_id\tgene_name\tgene_biotype\tgene_seq_start\tgene_seq_end\tseq_name\tseq_strand\tseq_coord_system\tdescription\tgene_id_version\n";
 
 open(TRANSCRIPT , ">ens_tx.txt");
-print TRANSCRIPT "tx_id\ttx_biotype\ttx_seq_start\ttx_seq_end\ttx_cds_seq_start\ttx_cds_seq_end\tgene_id\ttx_support_level\n";
+print TRANSCRIPT "tx_id\ttx_biotype\ttx_seq_start\ttx_seq_end\ttx_cds_seq_start\ttx_cds_seq_end\tgene_id\ttx_support_level\ttx_id_version\n";
 
 open(EXON , ">ens_exon.txt");
 print EXON "exon_id\texon_seq_start\texon_seq_end\n";
@@ -222,6 +224,7 @@ foreach my $gene_id (@gene_ids){
     if(!defined($gene_external_name)){
       $gene_external_name="";
     }
+    my $gene_id_version = $gene->stable_id_version;
     my $gene_biotype = $gene->biotype;
     my $gene_seq_start = $gene->start;
     my $gene_seq_end = $gene->end;
@@ -234,16 +237,7 @@ foreach my $gene_id (@gene_ids){
     foreach my $dbe (@{$all_entries}){
       print ENTREZGENE "$gene_id\t".$dbe->primary_id."\n";
     }
-    # my %entrezgene_hash=();
-    # foreach my $dbe (@{$all_entries}){
-    #   $entrezgene_hash{ $dbe->primary_id } = 1;
-    # }
-    # my $hash_size = keys %entrezgene_hash;
-    # my $entrezid = "";
-    # if($hash_size > 0){
-    #   $entrezid = join(";", keys %entrezgene_hash);
-    # }
-    print GENE "$gene_id\t$gene_external_name\t$gene_biotype\t$gene_seq_start\t$gene_seq_end\t$chrom\t$strand\t$coord_system\t$description\n";
+    print GENE "$gene_id\t$gene_external_name\t$gene_biotype\t$gene_seq_start\t$gene_seq_end\t$chrom\t$strand\t$coord_system\t$description\t$gene_id_version\n";
 
     ## process transcript(s)
     my @transcripts = @{ $gene->get_all_Transcripts };
@@ -268,6 +262,7 @@ foreach my $gene_id (@gene_ids){
 	$tx_cds_end = "NULL";
       }
       my $tx_id = $transcript->stable_id;
+      my $tx_id_version = $transcript->stable_id_version;
       my $tx_biotype = $transcript->biotype;
       my $tx_seq_start = $transcript->start;
       my $tx_seq_end = $transcript->end;
@@ -283,7 +278,7 @@ foreach my $gene_id (@gene_ids){
       # 	$tx_description = "NULL";
       # }
       ## write info.
-      print TRANSCRIPT "$tx_id\t$tx_biotype\t$tx_seq_start\t$tx_seq_end\t$tx_cds_start\t$tx_cds_end\t$gene_id\t$tx_tsl\n";
+      print TRANSCRIPT "$tx_id\t$tx_biotype\t$tx_seq_start\t$tx_seq_end\t$tx_cds_start\t$tx_cds_end\t$gene_id\t$tx_tsl\t$tx_id_version\n";
 ##      print G2T "$gene_id\t$tx_id\n";
 
       ## Process proteins/translations (if possible)
